==> org.knime.base.source_2.5.2.0032292.jar: org/knime/base/data/aggregation/general/ModeOperator.java <==
/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2011
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * -------------------------------------------------------------------
 */

package org.knime.base.data.aggregation.general;

import org.knime.core.data.DataCell;
import org.knime.core.data.DataType;
import org.knime.core.data.DataValue;
import org.knime.core.util.MutableInteger;

import org.knime.base.data.aggregation.AggregationOperator;
import org.knime.base.data.aggregation.GlobalSettings;
import org.knime.base.data.aggregation.OperatorColumnSettings;
import org.knime.base.data.aggregation.OperatorData;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * Returns the most frequent entry per group.
 *
 * @author Tobias Koetter, University of Konstanz
 */
public class ModeOperator extends AggregationOperator {

    private final Map<DataCell, MutableInteger> m_valCounter;

    /**Constructor for class ModeOperator.
     * @param globalSettings the global settings
     * @param opColSettings the operator column specific settings
     */
    public ModeOperator(final GlobalSettings globalSettings,
            final OperatorColumnSettings opColSettings) {
        this(new OperatorData("Mode", true, true, DataValue.class, true),
                globalSettings, opColSettings);
    }

    /**Constructor for class ModeOperator.
     * @param operatorData the operator data
     * @param globalSettings the global settings
     * @param opColSettings the operator column specific settings
     */
    protected ModeOperator(final OperatorData operatorData,
            final GlobalSettings globalSettings,
            final OperatorColumnSettings opColSettings) {
        super(operatorData, globalSettings, opColSettings);
        try {
            m_valCounter =
                new LinkedHashMap<DataCell, MutableInteger>(
                        getMaxUniqueValues());
        } catch (final OutOfMemoryError e) {
            throw new IllegalArgumentException(
                    "Maximum unique values number to big");
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected DataType getDataType(final DataType origType) {
        return origType;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AggregationOperator createInstance(
            final GlobalSettings globalSettings,
            final OperatorColumnSettings opColSettings) {
        return new ModeOperator(globalSettings, opColSettings);
    }


    /**
     * {@inheritDoc}
     */
    @Override
    protected boolean computeInternal(final DataCell cell) {
        MutableInteger counter = m_valCounter.get(cell);
        if (counter == null) {
            //check if the maps contains more values than allowed
            //before adding a new value
            if (m_valCounter.size() >= getMaxUniqueValues()) {
                setSkipMessage("Group contains to many unique values");
                return true;
            }
            counter = new MutableInteger(0);
            m_valCounter.put(cell, counter);
        }
        counter.inc();
        return false;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected DataCell getResultInternal() {
        if (m_valCounter.size() < 1) {
            return DataType.getMissingCell();
        }
        //get the cell with the most counts
        final Set<Entry<DataCell, MutableInteger>> entries =
            m_valCounter.entrySet();
        int max = Integer.MIN_VALUE;
        DataCell result = null;
        for (final Entry<DataCell, MutableInteger> entry : entries) {
            if (entry.getValue().intValue() > max) {
                max = entry.getValue().intValue();
                result = entry.getKey();
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void resetInternal() {
        m_valCounter.clear();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getDescription() {
        return "Takes the value with the most occurrences per group. "
        + "If multiple values match this criterion the first occurrence is "
        + "written to the output table.";
    }
}

==> org.knime.base.source_2.7.2.0036614.jar: org/knime/base/data/aggregation/general/ModeOperator.java <==
/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2011
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * -------------------------------------------------------------------
 */

package org.knime.base.data.aggregation.general;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.knime.base.data.aggregation.AggregationOperator;
import org.knime.base.data.aggregation.GlobalSettings;
import org.knime.base.data.aggregation.OperatorColumnSettings;
import org.knime.base.data.aggregation.OperatorData;
import org.knime.core.data.DataCell;
import org.knime.core.data.DataType;
import org.knime.core.data.DataValue;
import org.knime.core.util.MutableInteger;

/**
 * Returns the most frequent entry per group.
 *
 * @author Tobias Koetter, University of Konstanz
 */
public class ModeOperator extends AggregationOperator {

    private final Map<DataCell, MutableInteger> m_valCounter;

    /**Constructor for class ModeOperator.
     * @param globalSettings the global settings
     * @param opColSettings the operator column specific settings
     */
    public ModeOperator(final GlobalSettings globalSettings,
            final OperatorColumnSettings opColSettings) {
        this(new OperatorData("Mode", true, true, DataValue.class, true),
                globalSettings, opColSettings);
    }

    /**Constructor for class ModeOperator.
     * @param operatorData the operator data
     * @param globalSettings the global settings
     * @param opColSettings the operator column specific settings
     */
    protected ModeOperator(final OperatorData operatorData,
            final GlobalSettings globalSettings,
            final OperatorColumnSettings opColSettings) {
        super(operatorData, globalSettings, opColSettings);
        try {
            m_valCounter =
                new LinkedHashMap<DataCell, MutableInteger>(
                        getMaxUniqueValues());
        } catch (final OutOfMemoryError e) {
            throw new IllegalArgumentException(
                    "Maximum unique values number to big");
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected DataType getDataType(final DataType origType) {
        return origType;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AggregationOperator createInstance(
            final GlobalSettings globalSettings,
            final OperatorColumnSettings opColSettings) {
        return new ModeOperator(getOperatorData(), globalSettings, opColSettings);
    }


    /**
     * {@inheritDoc}
     */
    @Override
    protected boolean computeInternal(final DataCell cell) {
        MutableInteger counter = m_valCounter.get(cell);
        if (counter == null) {
            //check if the maps contains more values than allowed
            //before adding a new value
            if (m_valCounter.size() >= getMaxUniqueValues()) {
                setSkipMessage("Group contains to many unique values");
                return true;
            }
            counter = new MutableInteger(0);
            m_valCounter.put(cell, counter);
        }
        counter.inc();
        return false;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected DataCell getResultInternal() {
        if (m_valCounter.size() < 1) {
            return DataType.getMissingCell();
        }
        //get the cell with the most counts
        final Set<Entry<DataCell, MutableInteger>> entries =
            m_valCounter.entrySet();
        int max = Integer.MIN_VALUE;
        DataCell result = null;
        for (final Entry<DataCell, MutableInteger> entry : entries) {
            if (entry.getValue().intValue() > max) {
                max = entry.getValue().intValue();
                result = entry.getKey();
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void resetInternal() {
        m_valCounter.clear();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getDescription() {
        return "Takes the value with the most occurrences per group. "
        + "If multiple values match this criterion the first occurrence is "
        + "written to the output table.";
    }
}
